import * as Sentry from '@sentry/node'
import type { Express } from 'express'
import { environment } from './environment'

export const setupErrorTracking = (app: Express) => {
  Sentry.init({
    dsn: environment.SENTRY_DNS,
    environment: environment.SENTRY_ENVIRONMENT,
    enabled:
      ['staging', 'production'].includes(environment.SENTRY_ENVIRONMENT) &&
      process.env.NODE_ENV === 'production',
    integrations: [
      // enable Express.js middleware tracing
      new Sentry.Integrations.Express({
        // to trace all requests to the default router
        app,
        // alternatively, you can specify the routes you want to trace:
        // router: someRouter,
      }),
    ],

    // We recommend adjusting this value in production, or using tracesSampler
    // for finer control
    tracesSampleRate: 0.25,
  })

  return {
    middlewares: () => [Sentry.Handlers.requestHandler(), Sentry.Handlers.tracingHandler()],
    errorHandler: () => Sentry.Handlers.errorHandler(),
  }
}
