import { container } from '@/core/api/di/container'
import { AdminGetMyProfileUseCase } from '@/libs/application/use-cases/admin-auth/get-my-profile.use-case'
import { UnauthorizedException } from '@/libs/domain/errors'
import { NextFunction, Request, Response } from 'express'

const getMyProfileUseCase = container.get(AdminGetMyProfileUseCase)

export const adminAuthMiddleware = async (req: Request, res: Response, next: NextFunction) => {
  try {
    const getToken = (header = '') => header.split(' ').pop()
    const token = getToken(req.header('authorization'))

    if (!token) {
      throw new UnauthorizedException()
    }

    await getMyProfileUseCase.run({ token })

    return next()
  } catch (error) {
    return next(error)
  }
}
