import { NextFunction, Request, Response } from 'express'
import { RequestContext } from '@/libs/domain/contexts/request.context'
import { ParametersValidationSchema } from '../validation-schemas/parameters.validation-schema'
import {
  EntityType,
  ValidateIdExistsUseCase,
} from '@/libs/application/use-cases/shared/validate-id-exists.use-case'
import { container } from '../di/container'

const validateIdExistsUseCase = container.get(ValidateIdExistsUseCase)

export const validateIdExistsMiddleware = (entity: EntityType) => [
  ParametersValidationSchema.id,
  async (req: Request, res: Response, next: NextFunction) => {
    try {
      const { id } = req.params

      await validateIdExistsUseCase.run(
        {
          entity,
          id,
        },
        RequestContext.fromPrimitives(req)
      )

      return next()
    } catch (error) {
      return next(error)
    }
  },
]
