import { ValidateTransportCompanyEmailUseCase } from '@/libs/application/use-cases/transport-company/validate-email.use-case'
import { RequestContext } from '@/libs/domain/contexts/request.context'
import { NextFunction, Request, Response } from 'express'

import _get from 'lodash/get'
import { container } from '../di/container'

const validateTransportCompanyEmailUseCase = container.get(ValidateTransportCompanyEmailUseCase)

export const validateTransportCompanyEmailMiddleware =
  (path = 'email') =>
  async (req: Request, res: Response, next: NextFunction) => {
    try {
      const email = _get(req.body, path)

      await validateTransportCompanyEmailUseCase.run(email, RequestContext.fromPrimitives(req))

      return next()
    } catch (error) {
      return next(error)
    }
  }
