import { RequestContext } from '@/libs/domain/contexts/request.context'
import { NextFunction, Request, Response } from 'express'

import _get from 'lodash/get'
import { container } from '../di/container'
import { ValidateTransportCompanyRutUseCase } from '@/libs/application/use-cases/transport-company/validate-rut-use-case'

const validateTransportCompanyRutUseCase = container.get(ValidateTransportCompanyRutUseCase)

export const validateTransportCompanyRutMiddleware =
  (path = 'rut') =>
  async (req: Request, res: Response, next: NextFunction) => {
    try {
      const rut = _get(req.body, path)

      await validateTransportCompanyRutUseCase.run(rut, RequestContext.fromPrimitives(req))

      return next()
    } catch (error) {
      return next(error)
    }
  }
