import Router from 'express-promise-router'
import { authMiddleware } from '../middlewares/auth.middleware'
import { Role } from '@/libs/infrastructure/persistance/seeders/dev.seeder'
import { validateUserEmailMiddleware } from '@/core/api/middlewares/validate-user-email.middleware'
import { DriverController } from '../controllers/driver.controller'
import { DriverValidationSchema } from '../validation-schemas/driver.validation-schema'
import { validateUserRutMiddleware } from '@/core/api/middlewares/validate-user-rut.middleware'
import { ParametersValidationSchema } from '@/core/api/validation-schemas/parameters.validation-schema'
import { validateIdExistsMiddleware } from '@/core/api/middlewares/validate-id-exists.middleware'

export const driverRouter = Router()
  .use(authMiddleware([Role.adminEmployee, Role.vendorEmployee]))
  .post(
    '/',
    DriverValidationSchema.create,
    validateUserEmailMiddleware('user.email'),
    validateUserRutMiddleware('user.rut'),
    DriverController.create
  )
  .use('/:id', validateIdExistsMiddleware('driver'))
  .patch('/:id', DriverValidationSchema.update, DriverController.update)
  .delete('/:id', DriverController.delete)
  .get('/:id', DriverController.show)
  .get('/', ParametersValidationSchema.offsetCriteria, DriverController.list)
