import Router from 'express-promise-router'
import { authMiddleware } from '../middlewares/auth.middleware'
import { Role } from '@/libs/infrastructure/persistance/seeders/dev.seeder'
import { TransportCompanyController } from '../controllers/transport-company.controller'
import { TransportCompanyValidationSchema } from '../validation-schemas/transport-company.validation-schema'
import { validateTransportCompanyEmailMiddleware } from '../middlewares/validate-transport-company-email.middleware'
import { validateTransportCompanyRutMiddleware } from '../middlewares/validate-transport-company-rut.middleware'
import { ParametersValidationSchema } from '@/core/api/validation-schemas/parameters.validation-schema'
import { validateIdExistsMiddleware } from '@/core/api/middlewares/validate-id-exists.middleware'

export const transportCompanyRouter = Router()
  .use(authMiddleware([Role.adminEmployee, Role.vendorEmployee]))
  .post(
    '/',
    TransportCompanyValidationSchema.create,
    validateTransportCompanyEmailMiddleware('email'),
    validateTransportCompanyRutMiddleware('rut'),
    TransportCompanyController.create
  )
  .use('/:id', validateIdExistsMiddleware('transport-company'))
  .patch('/:id', TransportCompanyValidationSchema.update, TransportCompanyController.update)
  .delete('/:id', TransportCompanyController.delete)
  .get('/:id', TransportCompanyController.show)
  .get('/', ParametersValidationSchema.offsetCriteria, TransportCompanyController.list)
