import { environment } from '@/core/api/environment'
import * as cheerio from 'cheerio'

import mailhog from 'mailhog'
import { TimingHelper } from './timing.helper'

export class MailHelper {
  static async getResetTokenFromEmail(email: string) {
    await TimingHelper.wait() // for bull queue time
    const message = await mailHogApi.latestTo(email)
    if (!message) {
      throw new Error('email not found')
    }
    await mailHogApi.deleteMessage(message.ID)

    const $ = cheerio.load(message.html)

    const resetUrl = $('a').attr('href')

    if (!resetUrl) {
      throw new Error('reset url not found')
    }
    const resetToken = resetUrl.split('reset-password/')[1]

    return resetToken
  }

  static async getResetOTPCodeFromEmail(email: string) {
    await TimingHelper.wait() // for bull queue time
    const message = await mailHogApi.latestTo(email)
    if (!message) {
      throw new Error('email not found')
    }
    await mailHogApi.deleteMessage(message.ID)

    const $ = cheerio.load(message.html)

    const line = $('span:contains("Your OTP code is ")').text()

    const otpCode = line.split('is ')[1]

    return otpCode
  }

  static async getVerifyLinkFromEmail(email: string) {
    await TimingHelper.wait() // for bull queue time
    const message = await mailHogApi.latestTo(email)
    if (!message) {
      throw new Error('email not found')
    }
    await mailHogApi.deleteMessage(message.ID)

    const $ = cheerio.load(message.html)

    const resetUrl = $('a').attr('href')

    return resetUrl
  }
}

export const mailHogApi = mailhog({
  host: environment.MAIL_HOST,
  port: Number(environment.MAIL_HOG_WEB_PORT),
})
