import { container } from '@/core/api/di/container'
import { CustomerRepository } from '@/libs/domain/repositories/tenant/customer.repository'
import { DriverRepository } from '@/libs/domain/repositories/tenant/driver.repository'
import { TransportCompanyRepository } from '@/libs/domain/repositories/tenant/transport-company.repository'
import { UserRepository } from '@/libs/domain/repositories/tenant/user.repository'
import { CustomWorld } from '../custom-world'
import { Given, Then } from '@cucumber/cucumber'
import { OrderRepository } from '@/libs/domain/repositories/tenant/order.repository'
import { OrderStatus } from '@/libs/domain/entities/tenant/_types'
import { TenantCategoryRepository } from '@/libs/domain/repositories/master/tenant-category.repsitory'
import { DefaultFieldRepository } from '@/libs/domain/repositories/master/default-field.repository'

const customerRepository = container.get(CustomerRepository)
const userRepository = container.get(UserRepository)
const driverRepository = container.get(DriverRepository)
const transportCompanyRepository = container.get(TransportCompanyRepository)
const orderRepository = container.get(OrderRepository)
const tenantCategoryRepository = container.get(TenantCategoryRepository)
const defaultFieldRepository = container.get(DefaultFieldRepository)

const createEntity = async (entity: string, payload: any, tenantId: string) => {
  // update or save maybe, depends if "id" in payload exists
  switch (entity) {
    case 'user':
      return await userRepository.save(
        {
          rut: 'user_0000123456789',
          password: '12345', // not hashed
          email: 'some_user@gmail.com',
          firstName: 'some_firstName',
          lastName: 'some_lastName',
          phoneNumber: '9999999999999',
          isActive: true,
          ...payload,
        },
        tenantId!
      )

    case 'customer':
      return await customerRepository.save(
        {
          ...payload,
        },
        tenantId!
      )

    case 'driver':
      return await driverRepository.save({ ...payload }, tenantId!)

    case 'transport company':
      return transportCompanyRepository.save(
        {
          rut: 'transport_company_0000123456789',
          legalAddress: 'some legal address',
          legalName: 'some legal name',
          phoneNumber: '8888888888888',
          email: 'some_transportCompany@gmail.com',
          ...payload,
        },
        tenantId!
      )

    case 'order':
      return await orderRepository.save(
        {
          status: OrderStatus.requested,
          ...payload,
        },
        tenantId!
      )

    case 'tenant category':
      return await tenantCategoryRepository.save({
        description: 'lorem ipsum description',
        name: 'lorem ipsum name',
        ...payload,
      })

    case 'default field':
      return await defaultFieldRepository.save({
        label: 'some label',
        placeholder: 'some placeholder',
        span: 8,
        order: 123,
        type: 'text',
        ...payload,
      })

    default:
      throw new Error(`Creation of entity "${entity}" is not supported.`)
  }
}

Given(
  'the system has a {string} with:',
  async function (this: CustomWorld, entity: string, payload: string) {
    await createEntity(entity, JSON.parse(payload), this.tenantId!)
  }
)

Given(
  'the system has a {string} with id {string}',
  async function (this: CustomWorld, entity: string, id: string) {
    await createEntity(entity, { id }, this.tenantId!)
  }
)

Then(
  'the system cannot find the deleted {string} with id {string}',
  async function (this: CustomWorld, entity: string, id: string) {
    switch (entity) {
      case 'user':
        return await userRepository.findOne({ id }, this.tenantId!)

      case 'customer':
        return await customerRepository.findOne({ id }, this.tenantId!)

      case 'driver':
        return await driverRepository.findOne({ id }, this.tenantId!)

      case 'transport company':
        return transportCompanyRepository.findOne({ id }, this.tenantId!)

      case 'tenant category':
        return await tenantCategoryRepository.findOne({ id })
      case 'default field':
        return await defaultFieldRepository.findOne({ id })

      default:
        throw new Error(`Creation of entity "${entity}" is not supported.`)
    }
  }
)
