import { Injectable } from '@/core/api/di/decorators'
import { SuperAdminEntity } from '@/libs/domain/entities/master/super-admin.entity'
import { BadRequestException, UnauthorizedException } from '@/libs/domain/errors'
import { TokenHelper } from '@/libs/domain/helpers/token.helper'
import { SuperAdminRepository } from '@/libs/domain/repositories/master/super-admin.repository'

@Injectable()
export class AdminGetMyProfileUseCase {
  constructor(private readonly superAdminRepository: SuperAdminRepository) {}

  async run({ token }: { token: string }): Promise<SuperAdminEntity> {
    const payload = await TokenHelper.parseJWT(token ?? '')
    if (!payload?.id) {
      throw new UnauthorizedException()
    }
    const superAdmin = await this.superAdminRepository.findOne({
      id: payload?.id,
    })

    if (!superAdmin) {
      throw new UnauthorizedException()
    }
    if (!superAdmin.isActive) {
      throw new BadRequestException('Usuario deshabilitado.')
    }

    // @ts-ignore
    delete superAdmin?.password

    return superAdmin
  }
}
