import { Injectable } from '@/core/api/di/decorators'
import { environment } from '@/core/api/environment'
import { RequestContext } from '@/libs/domain/contexts/request.context'
import { BadRequestException } from '@/libs/domain/errors'
import { TokenHelper } from '@/libs/domain/helpers/token.helper'
import { MailMessageBuilder } from '@/libs/domain/notification/mail-message-builder'

import { UserRepository } from '@/libs/domain/repositories/tenant/user.repository'
import { MailerService } from '@/libs/domain/services/mailer.service'

@Injectable()
export class ForgotPasswordUseCase {
  constructor(
    private readonly userRepository: UserRepository,
    private readonly mailerService: MailerService
  ) {}

  async run({ email }: { email: string }, { tenant }: RequestContext): Promise<void> {
    const user = await this.userRepository.findOne(
      {
        email,
      },
      tenant.id
    )

    if (!user) {
      throw new BadRequestException('Correo invalido')
    }

    const minutes = 10
    const token = await TokenHelper.generateJWT({ id: user.id }, { expiresIn: `${minutes}m` })

    const resetPasswordUrl = `${environment.BACKOFFICE_URL}/reset-password/${token}`

    await this.mailerService.sendEmail(
      new MailMessageBuilder({
        from: `${environment.API_NAME} <${environment.API_EMAIL}>`,
        to: email,
        subject: 'Restablecer contraseña',
      })
        .addTitle(`Estimado ${user.firstName} ${user.lastName}`)
        .addLine(
          'Usted está recibiendo este correo electrónico porque hemos recibido una solicitud de restablecimiento de contraseña para su cuenta.'
        )
        .addAction('Restablecer contraseña', resetPasswordUrl)
        .addLine(`Este enlace de restablecimiento de contraseña caducará en ${minutes} minutos.`)
        .addLine(
          `Si no solicitó un restablecimiento de contraseña, no se requiere ninguna acción adicional.`
        )
        .build()
    )
  }
}
