import { Injectable } from '@/core/api/di/decorators'
import { RequestContext } from '@/libs/domain/contexts/request.context'
import { DriverEntity } from '@/libs/domain/entities/tenant/driver.entity'
import { TransportCompanyEntity } from '@/libs/domain/entities/tenant/transport-company.entity'
import { EncryptHelper } from '@/libs/domain/helpers/encrypt.helper'
import { DriverRepository } from '@/libs/domain/repositories/tenant/driver.repository'
import { UserRepository } from '@/libs/domain/repositories/tenant/user.repository'

@Injectable()
export class UpdateDriverUseCase {
  constructor(
    private readonly driverRepository: DriverRepository,
    private readonly userRepository: UserRepository
  ) {}

  async run(
    id: string,
    { transportCompanyId, user: updatedUser, ...payload }: DriverEntity,
    { tenant }: RequestContext
  ): Promise<DriverEntity> {
    const driver = await this.driverRepository.findOne({ id }, tenant.id)

    if (updatedUser?.password) {
      updatedUser.password = await EncryptHelper.hash(updatedUser.password)
    }

    await this.userRepository.save({ id: driver?.user?.id, ...updatedUser }, tenant.id) // cascade mode disabled

    if (transportCompanyId) {
      ;(payload.transportCompany as Partial<TransportCompanyEntity>) = {
        id: transportCompanyId,
      }
    }
    return await this.driverRepository.save(
      {
        id,
        ...payload,
      },
      tenant.id
    )
  }
}
