import { CursorPagination } from './cursor-pagination'
import { Filter, Filters } from './filters'
import { OffsetPagination } from './offset-pagination'
import { Order } from './order'

export class CursorCriteria {
  filters: Filters
  pagination: CursorPagination

  constructor({ filters, pagination }: { filters: Filters; pagination: CursorPagination }) {
    this.filters = filters
    this.pagination = pagination
  }

  hasFilters(): boolean {
    return Boolean(this.filters.value.length)
  }

  hasCursor(): boolean {
    return Boolean(this.pagination.cursor)
  }

  addFilter(filter: Filter) {
    this.filters.value.push(filter)
  }
}

export class OffsetCriteria {
  filters: Filters
  order: Order
  pagination: OffsetPagination

  constructor({
    filters,
    order,
    pagination,
  }: {
    filters: Filters
    order: Order
    pagination: OffsetPagination
  }) {
    this.filters = filters
    this.order = order
    this.pagination = pagination
  }

  hasFilters(): boolean {
    return Boolean(this.filters.value.length)
  }

  hasOrder(): boolean {
    return Boolean(this.order?.orderBy && this.order.orderType)
  }

  addFilter(filter: Filter) {
    this.filters.value.push(filter)
  }
}
