import { ParseHelper } from '../helpers/parse.helper'

export enum FilterOperator {
  EQUAL = 'EQUAL',
  NOT_EQUAL = 'NOT_EQUAL',
  GT = 'GT',
  GTE = 'GTE',
  LT = 'LT',
  LTE = 'LTE',
  CONTAINS = 'CONTAINS',
  NOT_CONTAINS = 'NOT_CONTAINS',
  ANY_OF = 'ANY_OF',
  IS_NULL = 'IS_NULL',
  IS_NOT_NULL = 'IS_NOT_NULL',
}

// eslint-disable-next-line sort-exports/sort-exports
export type Filter = {
  value?: string | number | Date | boolean | (string | number | boolean | Date)[]
  field: string
  operator: FilterOperator
}

export class Filters {
  value: Filter[]

  private constructor(filters: Filter[]) {
    this.value = filters
  }

  static fromPrimitives({ filters: primitiveFilters = [] }: any, allowedFields: string[]) {
    if (!Array.isArray(primitiveFilters)) {
      throw new Error('`filters` field in `query` object must be an array.')
    }

    const parsedFilters = (primitiveFilters as Filter[])
      .filter(filter => {
        if (!filter.field || !filter.operator) {
          return false
        }

        if (!FilterOperator[filter.operator as keyof typeof FilterOperator]) {
          return false
        }

        if ([FilterOperator.IS_NOT_NULL, FilterOperator.IS_NULL].includes(filter.operator)) {
          return true
        }

        if (!filter.value) {
          return false
        }

        if (Array.isArray(filter.value) && filter.value.length === 0) {
          return false
        }

        if (Array.isArray(filter.value) && filter.value.some(ParseHelper.isNotPrimitive)) {
          return false
        }

        if (Array.isArray(filter.value) && !ParseHelper.areSameType(filter.value)) {
          return false
        }

        if (!Array.isArray(filter.value) && !ParseHelper.isPrimitive(filter.value)) {
          return false
        }
        return true
      })

      // parse string value to Date object
      .map(filter => ({
        ...filter,
        value: Array.isArray(filter.value)
          ? filter.value.map(ParseHelper.parsePrimitive)
          : ParseHelper.parsePrimitive(filter.value),
      }))

    const _filters = new Filters(parsedFilters)

    _filters.applyWhiteList(allowedFields)

    return _filters
  }

  applyWhiteList(whiteList: string[]) {
    this.value = this.value.filter(filter => whiteList.includes(filter.field))
  }
}
