import { environment } from '@/core/api/environment'
import jwt, { JwtPayload, SignOptions } from 'jsonwebtoken'

export class TokenHelper {
  static generateJWT = (payload: any, options: SignOptions = {}): Promise<string> => {
    return new Promise((resolve, reject) => {
      jwt.sign(payload, environment.JWT_SECRET_KEY, options, (err, token) => {
        if (err) {
          reject(err)
        }
        resolve(token as string)
      })
    })
  }

  static parseJWT = (token: string): Promise<JwtPayload | null> => {
    return new Promise(resolve => {
      jwt.verify(token, environment.JWT_SECRET_KEY, function (err, decoded) {
        if (err) {
          resolve(null)
        }

        resolve(decoded as any)
      })
    })
  }
}
