import winston from 'winston'

winston.addColors({
  error: 'red',
  warn: 'yellow',
  info: 'green',
  http: 'magenta',
  debug: 'white',
})

export const logger = winston.createLogger({
  level: 'info',
  levels: {
    error: 0,
    warn: 1,
    info: 2,
    http: 3,
    verbose: 4,
    debug: 5,
    silly: 6,
  },
  silent: process.env.NODE_ENV === 'production',
  format: winston.format.json(),
  defaultMeta: {},
  transports: [],
})

if (process.env.NODE_ENV !== 'production') {
  logger.add(
    // development
    new winston.transports.Console({
      level: 'debug',
      format: winston.format.combine(
        // Add the message timestamp with the preferred format
        winston.format.timestamp({ format: 'YYYY-MM-DD HH:mm:ss:ms' }),
        // Tell Winston that the logs must be colored
        winston.format.colorize(),
        // Define the format of the message showing the timestamp, the level and the message
        winston.format.printf(
          ({ level, message, timestamp, ...metadata }) =>
            `${timestamp} ${level}: ${message} ${
              Object.values(metadata).length > 0 ? JSON.stringify(metadata, null, 2) : ''
            }`
        )
      ),
      // format: winston.format.combine(winston.format.colorize(), winston.format.simple()),
    })
  )
  // .add(
  //   new winston.transports.File({
  //     filename: 'combined.log',
  //     level: 'info',
  //   })
  // )

  // TODO: info level must be send to some event tracking tool or any database (production)
}
