import { DeepPartial } from 'utility-types'
import { CursorCriteria, OffsetCriteria } from '../../criteria/criteria'
import { BaseTenantEntity } from '../../entities/tenant/_base-tenant.entity'

export abstract class BaseTenantRepository<Entity extends BaseTenantEntity> {
  abstract save(entity: DeepPartial<Entity>, tenantId: string): Promise<Entity>

  abstract delete(entity: DeepPartial<Entity>, tenantId: string): Promise<void>

  abstract findOne(entity: DeepPartial<Entity>, tenantId: string): Promise<Entity | null>

  abstract findOneOrCreate(
    where: DeepPartial<Entity>,
    entity: DeepPartial<Entity>,
    tenantId: string
  ): Promise<Entity>

  abstract find(tenantId: string): Promise<Entity[]>

  abstract findByCursorCriteria(
    criteria: CursorCriteria,
    tenantId: string
  ): Promise<{ cursor: string | null; results: Entity[] }>

  abstract findByOffsetCriteria(
    criteria: OffsetCriteria,
    tenantId: string
  ): Promise<{ total: number; results: Entity[] }>
}
