import { EntitySchema, EntitySchemaColumnOptions, EntitySchemaOptions } from 'typeorm'
import { BaseTenantEntity } from '@/libs/domain/entities/tenant/_base-tenant.entity'

import { produce } from 'immer'

const toSnakeCase = (text: string) => text.replace(/[A-Z]/g, m => `_${m.toLowerCase()}`)

export class BaseTenantEntitySchema<T extends BaseTenantEntity> extends EntitySchema<T> {
  constructor(options: EntitySchemaOptions<T>) {
    super(
      produce(options, draft => {
        draft.schema = 'tenant'

        // @ts-ignore
        draft.columns.id = {
          type: 'uuid',
          generated: 'uuid',
          primary: true,
        } as EntitySchemaColumnOptions

        // @ts-ignore
        draft.columns.updatedAt = {
          type: 'timestamptz',
          precision: 3,
          updateDate: true,
        } as EntitySchemaColumnOptions

        // @ts-ignore
        draft.columns.createdAt = {
          type: 'timestamptz',
          precision: 3,
          createDate: true,
        } as EntitySchemaColumnOptions

        // @ts-ignore
        draft.columns.tenantId = {
          type: 'uuid',
        } as EntitySchemaColumnOptions

        draft.indices = (draft.indices ?? []).concat([
          {
            name: `${toSnakeCase(options.name)}` + `_TENANT_ID`,
            columns: ['tenantId'],
          },
        ])

        // make easy to define unique column at entity schema level (validation with tenantId column)
        const uniquesColumns = (
          Object.values(draft.columns) as Array<keyof typeof draft.columns>
        ).filter(column => Boolean((draft.columns[column] as EntitySchemaColumnOptions)?.unique))

        uniquesColumns.forEach(column => {
          delete (draft.columns[column] as EntitySchemaColumnOptions).unique
        })

        draft.uniques = (draft.uniques ?? [])?.concat(
          ...(uniquesColumns as string[]).map(column => ({
            name: `UNIQUE_${toSnakeCase(column)}_BY_TENANT`,
            columns: [column, 'tenantId'],
          }))
        )
      })
    )
  }
}
