import { OrderEntity } from '@/libs/domain/entities/tenant/order.entity'
import { BaseTenantEntitySchema } from './_base-tenant-schema'
import { OrderStatus } from '@/libs/domain/entities/tenant/_types'

export const orderSchema = new BaseTenantEntitySchema<OrderEntity>({
  name: 'order',
  columns: {
    price: {
      type: Number,
      nullable: true,
    },
    regularPrice: {
      type: Number,
      nullable: true,
    },
    status: {
      type: 'enum',
      enum: OrderStatus,
    },
    declineReason: {
      type: String,
      nullable: true,
    },
  },
  relations: {
    customer: {
      target: 'customer',
      type: 'many-to-one',
    },
    invoices: {
      target: 'invoice',
      type: 'one-to-many',
      inverseSide: 'order',
    },
    orderStatusHistories: {
      target: 'orderStatusHistory',
      type: 'one-to-many',
      inverseSide: 'order',
    },
    transportPayments: {
      target: 'transportPayment',
      type: 'one-to-many',
      inverseSide: 'order',
    },
    form: {
      target: 'form',
      type: 'one-to-one',
      inverseSide: 'order',
    },
    orderQuote: {
      target: 'orderQuote',
      type: 'one-to-one',
      inverseSide: 'order',
    },
    shippings: {
      target: 'shipping',
      type: 'one-to-many',
      inverseSide: 'order',
    },
  },
})
