import { UserEntity } from '@/libs/domain/entities/tenant/user.entity'

import { BaseTenantEntitySchema } from './_base-tenant-schema'

export const userSchema = new BaseTenantEntitySchema<UserEntity>({
  name: 'user',
  columns: {
    rut: {
      type: String,
      unique: true,
    },
    password: {
      type: String,
      select: false,
    },
    email: {
      type: String,
      unique: true,
    },
    firstName: {
      type: String,
    },
    lastName: {
      type: String,
    },
    phoneNumber: {
      type: String,
    },
    isActive: {
      type: 'boolean',
      default: false,
    },
  },
  relations: {
    roles: {
      type: 'many-to-many',
      target: 'role',
      joinTable: true,
    },
    customer: {
      target: 'customer',
      type: 'one-to-one',
      inverseSide: 'user',
    },
    employee: {
      target: 'employee',
      type: 'one-to-one',
      inverseSide: 'user',
    },
    driver: {
      target: 'driver',
      type: 'one-to-one',
      inverseSide: 'user',
    },
  },
})
