import { pugEngine } from 'nodemailer-pug-engine'
import { BaseQueue } from './_base-queue'
import nodemailer from 'nodemailer'
import { environment } from '@/core/api/environment'
import path from 'path'
import { MailSendPayload } from '@/libs/domain/services/mailer.service'
import { logger } from '../../domain/logger'

const transporter = nodemailer.createTransport({
  host: environment.MAIL_HOST,
  port: Number(environment.MAIL_PORT),
  auth: {
    user: environment.MAIL_USER,
    pass: environment.MAIL_PASSWORD,
  },
})

transporter.use(
  'compile',
  pugEngine({
    templateDir: path.join(__dirname, '/../views/mails'), // future review
  })
)

transporter.on('error', logger.debug)

export const mailerQueue = new BaseQueue('mailer')

mailerQueue.addWorker<MailSendPayload>(async job => {
  await transporter.sendMail(job.data)
})

mailerQueue.addEventListener('error', error => {
  logger.debug(error)
})
